#!/usr/bin/ruby
# Copyright (c) 2012, Comfirm AB
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

#     * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#     * Neither the name of the Comfirm AB nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

require 'rubygems'
require 'uri'
require 'net/http'
require 'json'

module AlphaMail
	# Email contact class
	class Contact
		attr_accessor :name
		attr_accessor :email

		def initialize(name = '', email = '')
			@name = name
			@email = email
		end
	end

	# Message payload class
	class MessagePayload
		attr_accessor :project_id
		attr_accessor :receiver_id
		attr_accessor :sender
		attr_accessor :receiver
		attr_accessor :body

		def initialize(project_id = 0, receiver_id = 0, sender = Contact.new('', ''), receiver = Contact.new('', ''), body = '')
			@project_id = project_id
			@receiver_id = receiver_id
			@sender = sender
			@receiver = receiver
			@body = body
			@body_json = ''
		end

		def to_json(*a)
			if @body.respond_to?('to_json')
				@body_json  = @body.to_json
			end
		
			{
				'project_id'		=>	project_id,
				'receiver_id'		=>	receiver_id,
				'sender_name'		=>	sender.name,
				'sender_email'		=>	sender.email,
				'receiver_name'		=>	receiver.name,
				'receiver_email'	=>	receiver.email,
				'body'			=>	@body_json
		
			}.to_json(*a)
		end
	end

	# Response class
	class Result
		attr_accessor :error_code
		attr_accessor :message
	
		def initialize(error_code = -1, message = 'Unknow error')
			@error_code = error_code
			@message = message
		end
	end

	# AlphaMail email service
	class EmailService
		def initialize(service_url, api_token)
			@service_url = service_url + '/email/queue'
			@api_token = api_token
		end

		def queue(payload = '')
			# Queue the message
			uri = URI.parse(@service_url)
			req = Net::HTTP::Post.new(uri.path)
			req.basic_auth '', @api_token
			req.body = payload.to_json
			req.content_type = 'application/json'
			res = Net::HTTP.start(uri.host, uri.port) {|http|
				http.request(req)
			}
		
			# Check response
			case res.code
				when '200' then
					am_res = JSON.parse(res.body)
					return Result.new(am_res['error_code'], am_res['message'])
				else
					am_res = JSON.parse(res.body)
					return Result.new(am_res['error_code'], am_res['message'])
			end 
			return Result.new
		end
	end
end


